// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-

const Shell = imports.gi.Shell;
const Lang = imports.lang;
const Signals = imports.signals;
const Gio = imports.gi.Gio;

const Main = imports.ui.main;

const AppRecord = new Lang.Class({
    Name: 'AppRecord',

    _init: function(settings, key) {
	this._key = key;
	this._settings = settings;
        this._record = {};
        this._settings.connect('changed::' + this._key, Lang.bind(this, this._onRensChanged));
        this.reload();
    },

    _onRensChanged: function() {
        this.reload();
        this.emit('changed');
    },

    reload: function() {
        let ids = this._settings.get_strv(this._key);
        let appSys = Shell.AppSystem.get_default();

        let apps = ids.map(function (id) {
                return appSys.lookup_app(id);
            }).filter(function (app) {
                return app != null;
            });
        this._record = {};
        for (let i = 0; i < apps.length; i++) {
            let app = apps[i];
            this._record[app.get_id()] = app;
        }
    },

    _getIds: function() {
        let ret = [];
	this.reload();
        for (let id in this._record)
	{
            ret.push(id);
	}
        return ret;
    },

    getRecordMap: function() {
        return this._record;
    },

    getRecord: function() {
        let ret = [];
        for (let id in this._record)
            ret.push(this._record[id]);
        return ret;
    },

    isRecord: function(appId) {
        return appId in this._record;
    },

    _addRecord: function(appId, pos) {
        if (appId in this._record)
            return false;

        let app = Shell.AppSystem.get_default().lookup_app(appId);

        if (!app)
            return false;

        let ids = this._getIds();
        if (pos == -1)
            ids.push(appId);
        else
            ids.splice(pos, 0, appId);
        this._settings.set_strv(this._key, ids);
        this._record[appId] = app;
        return true;
    },

    addRecordAtPos: function(appId, pos) {
        if (!this._addRecord(appId, pos))
            return;
    },

    addRecord: function(appId) {
        this.addRecordAtPos(appId, -1);
    },

    moveRecordToPos: function(appId, pos) {
        this._removeRecord(appId);
        this._addRecord(appId, pos);
    },

    _removeRecord: function(appId) {
        if (!appId in this._record)
            return false;

        let ids = this._getIds().filter(function (id) { return id != appId; });
        this._settings.set_strv(this._key, ids);
        return true;
    },

    removeRecord: function(appId) {
        if (!this._removeRecord(appId))
            return;
    }
});
Signals.addSignalMethods(AppRecord.prototype);
