const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const Gdm = imports.gi.Gdm;
const GLib = imports.gi.GLib;
const GObject = imports.gi.GObject;
const Gtk = imports.gi.Gtk;
const Shell = imports.gi.Shell;
const Lang = imports.lang;
const Signals = imports.signals;
const Meta = imports.gi.Meta;
const St = imports.gi.St;
const Mainloop = imports.mainloop;
const Atk = imports.gi.Atk;

const AppFavorites = imports.ui.appFavorites;
const DND = imports.ui.dnd;
const GrabHelper = imports.ui.grabHelper;
const IconGrid = imports.ui.iconGrid;
const Main = imports.ui.main;
const Overview = imports.ui.overview;
const OverviewControls = imports.ui.overviewControls;
const PopupMenu = imports.ui.popupMenu;
const Tweener = imports.ui.tweener;
const Workspace = imports.ui.workspace;
const Params = imports.misc.params;
const Util = imports.misc.util;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const LoginManager = imports.misc.loginManager;
const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
const _ = Gettext.gettext;

//Add kylin code 
const APP_IN_FIRST_AREA_BEGIN = 0;
const APP_IN_FIRST_AREA_RECENT = 1;
const APP_IN_DIR_AREA = 2;

const power_items = [ 'switch', 'logout', 'lock', 'poweroff', 'restart', 'suspend' ];
const DESKTOP_INTERFACE_SCHEMA = 'org.gnome.desktop.interface';
const KEY_TEXT_SCALING_FACTOR  = 'text-scaling-factor';

const AppItemMenu = new Lang.Class({
    Name: 'AppItemMenu',
    Extends: PopupMenu.PopupMenu,

    _init: function(source, button, area) {
	this._button = button;
	this._area = area;
        let side = St.Side.LEFT;
        if (Clutter.get_default_text_direction() == Clutter.TextDirection.RTL)
            side = St.Side.RIGHT;

        this.parent(source.actor, 0.0, side);

        // We want to keep the item hovered while the menu is up
        this.blockSourceEvents = true;

        this._source = source;

        this.actor.add_style_class_name('app-well-menu');
        //this.actor.add_style_class_name('second-menu');
	let settings = new Gio.Settings({ schema_id: DESKTOP_INTERFACE_SCHEMA });
	let factor = settings.get_double(KEY_TEXT_SCALING_FACTOR);
	this.actor.set_width(165 * factor);

        // Chain our visibility and lifecycle to that of the source
        source.actor.connect('notify::mapped', Lang.bind(this, function () {
            if (!source.actor.mapped)
                this.close();
        }));
        source.actor.connect('destroy', Lang.bind(this, this.destroy));

        Main.uiGroup.add_actor(this.actor);
    },

    _addToDesktop: function() {
        try {
            let app = this._source.app;
            let file = Gio.file_new_for_path(app.get_app_info().get_filename());
            let path = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DESKTOP);
            let destFile = Gio.file_new_for_path(path + "/" + app.get_id());
            file.copy(destFile, 0, null, function(){});
            // Need to find a way to do that using the Gio library, but modifying the access::can-execute attribute on the file object seems unsupported
            Util.spawnCommandLine("chmod +x \"" + path + "/" + app.get_id() + "\"");
            return true;
        } catch(e) {
            global.log(e);
        }
        return false;
    },

    //add to recent area
    _addToRecords: function() {
        let canRecord = global.settings.is_writable(this._button.RECENT_APPS_KEY);
	if (canRecord) {
	    let isRencent = this._button.appRecentRecordInstance.getAppRecords().isRecord(this._source.app.get_id());
	    if(isRencent){
                this._button.appRecentRecordInstance.moveRecordToPos(this._source.app.get_id(), 0);	
	    } else {
                this._button.appRecentRecordInstance.addRecordAtPos(this._source.app.get_id(), 0);	
	   }
	}
    },

    _redisplay: function() {
        this.removeAll();
        let windows = this._source.app.get_windows().filter(function(w) {
            return !w.skip_taskbar;
        });

        // Display the app windows menu items and the separator between windows
        // of the current desktop and other windows.
        let activeWorkspace = global.screen.get_active_workspace();
        let separatorShown = windows.length > 0 && windows[0].get_workspace() != activeWorkspace;

        if (!this._source.app.is_window_backed()) {
            this._newWindowMenuItem = this._appendMenuItem(_("Open"));
            this._newWindowMenuItem.connect('activate', Lang.bind(this, function() {
		    this._addToRecords();
		    this._button._reload();
                    this._source.app.open_new_window(-1);
		    this._button.menu.toggle();
                }));
/*
            this._adminWindowMenuItem = this._appendMenuItem(_("Run as administrator"));
            this._adminWindowMenuItem.connect('activate', Lang.bind(this, function() {
		    this._addToRecords();
                    this._source.app.open_new_window(-1);
		    this._button.menu.toggle();
                }));
            this._appendSeparator();
*/

            let canFavorite = global.settings.is_writable('favorite-apps');

            if (canFavorite) {
                this._appendSeparator();

                let isFavorite = AppFavorites.getAppFavorites().isFavorite(this._source.app.get_id());

                if (isFavorite) {
                } else {
                    let item = this._appendMenuItem(_("Pin to Taskbar"));
                    item.connect('activate', Lang.bind(this, function() {
                        let favs = AppFavorites.getAppFavorites();
                        favs.addFavorite(this._source.app.get_id());
                    }));
		 }
            }

            let canRecord = global.settings.is_writable(this._button.BEGIN_APPS_KEY);

            if (canRecord) {

                let isRecord = this._button.appBeginRecordInstance.isRecord(this._source.app.get_id());

                if (isRecord) {
		    if(this._area == APP_IN_FIRST_AREA_BEGIN)
		    {
                        let item = this._appendMenuItem(_("Unpin from Start Menu"));
                        item.connect('activate', Lang.bind(this, function() {
                            this._button.appBeginRecordInstance.removeRecord(this._source.app.get_id());
                            this._button.appRecentRecordInstance.removeRecord(this._source.app.get_id());
			    this._button._reload();
                        }));
		    }
                } else {
                    let item = this._appendMenuItem(_("Pin to Start Menu"));
                    item.connect('activate', Lang.bind(this, function() {
                        this._button.appBeginRecordInstance.addRecord(this._source.app.get_id());
			if(this._area == APP_IN_FIRST_AREA_RECENT) {
			    this._button._reload();
			}
                    }));
		 }
            }

	    let app = this._source.app;
            let path = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DESKTOP);
            let file = Gio.file_new_for_path(path + "/" + app.get_id());
            if (!file.query_exists(null)){
                this._appendSeparator();
                this._addToDesktopItem = this._appendMenuItem(_("Add to Desktop(create shortcut)"));
                this._addToDesktopItem.connect('activate', Lang.bind(this, function() {
                this._addToDesktop();
            	}));
            }

	    if(this._area == APP_IN_FIRST_AREA_BEGIN)
	    {
                this._appendSeparator();
                this._removeMenuItem = this._appendMenuItem(_("Remove from this list"));
                this._removeMenuItem.connect('activate', Lang.bind(this, function() {
                        this._button.appBeginRecordInstance.removeRecord(this._source.app.get_id());
                        this._button.appRecentRecordInstance.removeRecord(this._source.app.get_id());
			this._button._reload();
                    }));
	    }

	    if(this._area == APP_IN_FIRST_AREA_RECENT)
	    {
                this._appendSeparator();
                this._removeMenuItemRe = this._appendMenuItem(_("Remove from this list"));
                this._removeMenuItemRe.connect('activate', Lang.bind(this, function() {
	    		let isRencent = this._button.appRecentRecordInstance.isRecord(this._source.app.get_id());
	    		if(isRencent){
                	    this._button.appRecentRecordInstance.removeRecord(this._source.app.get_id());	
			    this._button._reload();
	    		}
                    }));
	    }

            if (Shell.AppSystem.get_default().lookup_app('org.gnome.Software.desktop')) {
                this._appendSeparator();
                let item = this._appendMenuItem(_("Show Details"));
                item.connect('activate', Lang.bind(this, function() {
                    let id = this._source.app.get_id();
                    let args = GLib.Variant.new('(ss)', [id, '']);
                    Gio.DBus.get(Gio.BusType.SESSION, null,
                        function(o, res) {
                            let bus = Gio.DBus.get_finish(res);
                            bus.call('org.gnome.Software',
                                     '/org/gnome/Software',
                                     'org.gtk.Actions', 'Activate',
                                     GLib.Variant.new('(sava{sv})',
                                                      ['details', [args], null]),
                                     null, 0, -1, null, null);
                            Main.overview.hide();
                        });
		    this._button.menu.toggle();
                }));
            }
        }
    },

    _appendSeparator: function () {
        let separator = new PopupMenu.PopupSeparatorMenuItem();
	separator.actor.remove_all_children();
	separator.actor.set_style_class_name('second-separator-menu-item');
        this.addMenuItem(separator);
    },

    _appendMenuItem: function(labelText) {
        // FIXME: app-well-menu-item style
        let item = new PopupMenu.PopupMenuItem(labelText, {style_class: ''});
	item.actor.remove_all_children();
	item.actor.add_style_class_name('second-menu-item');
	item.actor.label_actor.add_style_class_name('second-menu-item-label-app');
	item.actor.add_child(item.actor.label_actor);
        this.addMenuItem(item);
        return item;
    },

    popup: function(activatingButton) {
        this._redisplay();
        this.open();
    },
    
    popdown: function() {
        this.close(~0);
    }
});
Signals.addSignalMethods(AppItemMenu.prototype);

const PowerItemMenu = new Lang.Class({
    Name: 'PowerItemMenu',
    Extends: PopupMenu.PopupMenu,
    POWER_ITEMS_KEY: 'power-items',

    _init: function(source, button) {
	this._button = button;
        let side = St.Side.TOP;
        if (Clutter.get_default_text_direction() == Clutter.TextDirection.RTL)
            side = St.Side.RIGHT;

        this.parent(source.actor, -0.15, side);
        this.actor.add_style_class_name('app-well-menu');
        //this.actor.add_style_class_name('second-menu');
	let settings = new Gio.Settings({ schema_id: DESKTOP_INTERFACE_SCHEMA });
	let factor = settings.get_double(KEY_TEXT_SCALING_FACTOR);
	this.actor.set_width(120 * factor);

        // We want to keep the item hovered while the menu is up
        this.blockSourceEvents = true;

        this._source = source;


        // Chain our visibility and lifecycle to that of the source
        source.actor.connect('notify::mapped', Lang.bind(this, function () {
            if (!source.actor.mapped)
                this.close();
        }));
        source.actor.connect('destroy', Lang.bind(this, this.destroy));

        Main.uiGroup.add_actor(this.actor);
    },

    _showPowerItem: function (index) {
	let ids = this._button._settings.get_strv(this.POWER_ITEMS_KEY);
        if(ids.indexOf(power_items[index]) != -1)
	    return true;
	else
	    return false;
    },

    _redisplay: function() {
        this.removeAll();
        if (this._showPowerItem(0))
	{
            this._switchWindowMenuItem = this._appendMenuItem(_("Switch user"));
            this._switchWindowMenuItem.connect('activate', Lang.bind(this, function() {
	       this._button.menu.toggle();
	       if (Main.screenShield)
                    Main.screenShield.lock(false);
	       Clutter.threads_add_repaint_func_full(Clutter.RepaintFlags.POST_PAINT, () => {
            	    Gdm.goto_login_session_sync(null);
                    return false;
               });
            }));
	}
        if (this._showPowerItem(1))
	{
            this._logoutWindowMenuItem = this._appendMenuItem(_("Log Out"));
            this._logoutWindowMenuItem.connect('activate', Lang.bind(this, function() {
	        this._button._session.LogoutRemote(0);
	        this._button.menu.toggle();
            }));
	}
        if (this._showPowerItem(2)) 
  	{
            this._lockWindowMenuItem = this._appendMenuItem(_("Lock"));
            this._lockWindowMenuItem.connect('activate', Lang.bind(this, function() {
	        this._button.menu.toggle();
	        Main.screenShield.lock(true);
            }));
	}
	this._appendSeparator();
        if (this._showPowerItem(3)) 
	{
            this._shutdownWindowMenuItem = this._appendMenuItem(_("Power Off"));
            this._shutdownWindowMenuItem.connect('activate', Lang.bind(this, function() {
	        this._button.menu.toggle();
	        this._button._session.ShutdownRemote(0);
            }));
	}
        if (this._showPowerItem(4)) {
            this._restartWindowMenuItem = this._appendMenuItem(_("Restart"));
            this._restartWindowMenuItem.connect('activate', Lang.bind(this, function() {
	        this._button.menu.toggle();
	        this._button._session.RebootRemote(0);
            }));
	}
        if (this._showPowerItem(5)) 
	{
	    this._appendSeparator();
            this._suspendWindowMenuItem = this._appendMenuItem(_("Suspend"));
            this._suspendWindowMenuItem.connect('activate', Lang.bind(this, function() {
	        let loginManager = LoginManager.getLoginManager();
                loginManager.canSuspend(Lang.bind(this, function(result) {
                	if (result) {
                        loginManager.suspend();
                	}
       	        }));

	        this._button.menu.toggle();
            }));
	}
    },

    _appendSeparator: function () {
        let separator = new PopupMenu.PopupSeparatorMenuItem();
	separator.actor.remove_all_children();
	separator.actor.set_style_class_name('second-separator-menu-item');
        this.addMenuItem(separator);
    },

    _appendMenuItem: function(labelText) {
        // FIXME: app-well-menu-item style
        let item = new PopupMenu.PopupMenuItem(labelText, {style_class: ''});
	item.actor.remove_all_children();
	item.actor.add_style_class_name('second-menu-item');
	item.actor.label_actor.add_style_class_name('second-menu-item-label');
	item.actor.add_child(item.actor.label_actor);
	this.addMenuItem(item);
        return item;
    },

    popup: function(activatingButton) {
        this._redisplay();
        this.open();
    }
});
Signals.addSignalMethods(PowerItemMenu.prototype);
